import pandas as pd
import io
from PIL import Image
from torch.utils.data import Dataset
from torchvision import transforms
from aiia import AIIA

class UpscaleDataset(Dataset):
    def __init__(self, parquet_file, transform=None):
        self.df = pd.read_parquet(parquet_file, columns=['image_512', 'image_1024']).head(10000)
        self.transform = transform

    def __len__(self):
        return len(self.df)

    def __getitem__(self, idx):
        row = self.df.iloc[idx]
        # Decode the byte strings into images
        low_res_bytes = row['image_512']
        high_res_bytes = row['image_1024']
        low_res_image = Image.open(io.BytesIO(low_res_bytes)).convert('RGB')
        high_res_image = Image.open(io.BytesIO(high_res_bytes)).convert('RGB')
        if self.transform:
            low_res_image = self.transform(low_res_image)
            high_res_image = self.transform(high_res_image)
        return low_res_image, high_res_image

# Example transform: converting PIL images to tensors
transform = transforms.Compose([
    transforms.ToTensor(),
])


import torch

# Replace with your actual pretrained model path
pretrained_model_path = "/root/vision/AIIA/AIIA-base-512"  

# Load the model using the AIIA.load class method (the implementation copied in your query)
model = AIIA.load(pretrained_model_path)
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
model = model.to(device)
from torch import nn, optim
from torch.utils.data import DataLoader

# Create your dataset and dataloader
dataset = UpscaleDataset("/root/training_data/vision-dataset/image_upscaler.parquet", transform=transform)
data_loader = DataLoader(dataset, batch_size=16, shuffle=True)

# Define a loss function and optimizer
criterion = nn.MSELoss()
optimizer = optim.Adam(model.parameters(), lr=1e-4)

num_epochs = 10
model.train()  # Set model in training mode

for epoch in range(num_epochs):
    epoch_loss = 0.0
    for low_res, high_res in data_loader:
        low_res = low_res.to(device)
        high_res = high_res.to(device)

        optimizer.zero_grad()
        outputs = model(low_res)
        loss = criterion(outputs, high_res)
        loss.backward()
        optimizer.step()
        epoch_loss += loss.item()
    print(f"Epoch {epoch + 1}, Loss: {epoch_loss / len(data_loader)}")
    
# Optionally, save the finetuned model to a new directory
finetuned_model_path = "aiuNN"
model.save(finetuned_model_path)
